/*
 *  Copyright (C) 2003 Christian Meyer
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <partedpp/partedpp.h>

using namespace Ped;

/* ------- *
 * private *
 * ------- */

Disk::Disk(const PedDisk *disk)
    : disk_(ped_disk_duplicate(disk)) {
}

Disk& Disk::operator=(const Disk &disk) {
    if (this == &disk) {
        return *this;
    }
    disk_ = ped_disk_duplicate(disk.disk_);
    return *this;
}

const PedDisk* Disk::get_c_disk() const {
    return disk_;
}

PedDisk* Disk::get_c_disk() {
    return disk_;
}

/* ------ *
 * public *
 * ------ */

Disk::Disk(const Disk &disk)
    : disk_(ped_disk_duplicate(disk_)) {
}

Disk::Disk(Device &device)
    : disk_(ped_disk_new(device.get_c_device())) {
}

Disk::Disk(Device &device, DiskType &type)
    : disk_(ped_disk_new_fresh(device.get_c_device(),
                               type.get_c_disktype())) {
}

DiskType& Disk::probe(Device &dev) throw (std::runtime_error) {
    PedDiskType *disk_type_ = ped_disk_probe(dev.get_c_device());
    if (!disk_type_) {
        throw std::runtime_error("Could not probe device!");
    }
    disktype_ = std::auto_ptr<DiskType>(new DiskType(disk_type_));
    return *(disktype_.get());
}

bool Disk::clobber(Device &device) {
    return (ped_disk_clobber(device.get_c_device()));
}

bool Disk::clobber_exclude(Device &device, const DiskType &type) {
    return (ped_disk_clobber_exclude(device.get_c_device(), type.get_c_disktype()));
}

bool Disk::commit() {
    return (ped_disk_commit(disk_));
}

bool Disk::commit_to_dev() {
    return (ped_disk_commit_to_dev(disk_));
}

bool Disk::commit_to_os() {
    return (ped_disk_commit_to_os(disk_));
}

const bool Disk::check() const {
    return (ped_disk_check(disk_));
}

const void Disk::print() const {
    return (ped_disk_print(disk_));
}

bool Disk::add_partition(Partition &part,
                         const Constraint &constraint) {
    return (ped_disk_add_partition(disk_, part.get_c_partition(),
                                   constraint.get_c_constraint()));
}

bool Disk::remove_partition(Partition &part) {
    return (ped_disk_remove_partition(disk_, part.get_c_partition()));
}

bool Disk::delete_partition(Partition &part) {
    return (ped_disk_delete_partition(disk_, part.get_c_partition()));
}

bool Disk::delete_all() {
    return (ped_disk_delete_all(disk_));
}

bool Disk::set_partition_geom(Partition &part,
                              const Constraint &constraint,
                              const Sector start,
                              const Sector end) {
    return (ped_disk_set_partition_geom(disk_, part.get_c_partition(),
                                        constraint.get_c_constraint(),
                                        start, end));
}

bool Disk::maximize_partition(Partition &part,
                              const Constraint &constraint) {
    return (ped_disk_maximize_partition(disk_,
                                        part.get_c_partition(),
                                        constraint.get_c_constraint()));
}

Geometry& Disk::get_max_partition_geometry(Partition &part,
                                           Constraint &constraint)
                                           throw (std::runtime_error) {
    geometry_ = std::auto_ptr<PedGeometry>(ped_disk_get_max_partition_geometry(disk_,
                    part.get_c_partition(),
                    constraint.get_c_constraint()));
    if (!(geometry_.get())) {
        throw std::runtime_error("ped_disk_get_max_partition_geometry() failed!");
    }
    geom_ = std::auto_ptr<Geometry>(new Geometry(geometry_.get()));
    return *(geom_.get());
}

bool Disk::minimize_extended_partition() {
    return (ped_disk_minimize_extended_partition(disk_));
}

Partition& Disk::get_next_partition(Partition &part) const throw (std::runtime_error) {
    PedPartition *partition_ = ped_disk_next_partition(disk_, part.get_c_partition());
    if (!partition_) {
        throw std::runtime_error("No further partitions found!");
    }
    part_ = std::auto_ptr<Partition>(new Partition(partition_));
    return *(part_.get());
}

Partition& Disk::get_partition(int num) const throw (std::runtime_error) {
    PedPartition *partition_ = ped_disk_get_partition(disk_, num);
    if (!partition_) {
        throw std::runtime_error("No such partition found!");
    }
    part_ = std::auto_ptr<Partition>(new Partition(partition_));
    return *(part_.get());
}

Partition& Disk::get_partition_by_sector(Sector sect) const throw (std::runtime_error) {
    PedPartition *partition_ = ped_disk_get_partition_by_sector(disk_, sect);
    if (!partition_) {
        throw std::runtime_error("No such partition found!");
    }
    part_ = std::auto_ptr<Partition>(new Partition(partition_));
    return *(part_.get());
}

Partition& Disk::get_extended_partition() const throw (std::runtime_error) {
    PedPartition *partition_ = ped_disk_extended_partition(disk_);
    if (!partition_) {
        throw std::runtime_error("No extended partition could be found!");
    }
    part_ = std::auto_ptr<Partition>(new Partition(partition_));
    return *(part_.get());
}

const int Disk::get_primary_partition_count() const {
    return (ped_disk_get_primary_partition_count(disk_));
}

const int Disk::get_max_primary_partition_count() const {
    return (ped_disk_get_max_primary_partition_count(disk_));
}

const int Disk::get_last_partition_num() const {
    return (ped_disk_get_last_partition_num(disk_));
}

const Device& Disk::get_device() const {
    device_ = std::auto_ptr<Device>(new Device(disk_->dev));
    return *(device_.get());
}

const DiskType& Disk::get_disktype() const {
    disktype_ = std::auto_ptr<DiskType>(new DiskType(disk_->type));
    return *(disktype_.get());
}

Disk::~Disk() {
    if (disk_) {
        ped_disk_destroy(disk_);
        if (disk_) {
            free(disk_);
        }
    }
}

/* vim: set tabstop=4 et shiftwidth=4: */
